@@ -1,3 +1,7 @@
+#ifdef HAVE_CONFIG_H
+#include <config.h>
+#endif
+
 #include <glib.h>
 #include <glib/gstdio.h>
 #include <gio/gio.h>
@@ -6,8 +10,8 @@
 
 #include "server/e-ews-folder.h"
 
-#define S_LOCK(x) (g_static_rec_mutex_lock(&(x)->priv->s_lock))
-#define S_UNLOCK(x) (g_static_rec_mutex_unlock(&(x)->priv->s_lock))
+#define S_LOCK(x) (g_rec_mutex_lock(&(x)->priv->s_lock))
+#define S_UNLOCK(x) (g_rec_mutex_unlock(&(x)->priv->s_lock))
 
 #define STORE_GROUP_NAME "##storepriv"
 #define CURRENT_SUMMARY_VERSION 1
@@ -21,7 +25,7 @@ struct _CamelEwsStoreSummaryPrivate {
 	 * So entries must always be removed from fname_id_hash *first*. */
 	GHashTable *id_fname_hash;
 	GHashTable *fname_id_hash;
-	GStaticRecMutex s_lock;
+	GRecMutex s_lock;
 
 	GFileMonitor *monitor_delete;
 };
@@ -38,7 +42,7 @@ ews_store_summary_finalize (GObject *object)
 	g_free (priv->path);
 	g_hash_table_destroy (priv->fname_id_hash);
 	g_hash_table_destroy (priv->id_fname_hash);
-	g_static_rec_mutex_free (&priv->s_lock);
+	g_rec_mutex_clear (&priv->s_lock);
 	if (priv->monitor_delete)
 		g_object_unref (priv->monitor_delete);
 
@@ -72,7 +76,7 @@ camel_ews_store_summary_init (CamelEwsStoreSummary *ews_summary)
 		g_str_hash, g_str_equal,
 		(GDestroyNotify) g_free,
 		(GDestroyNotify) g_free);
-	g_static_rec_mutex_init (&priv->s_lock);
+	g_rec_mutex_init (&priv->s_lock);
 }
 
 static gchar *build_full_name (CamelEwsStoreSummary *ews_summary, const gchar *fid)
@@ -557,6 +561,21 @@ camel_ews_store_summary_set_foreign (CamelEwsStoreSummary *ews_summary,
 	S_UNLOCK (ews_summary);
 }
 
+void
+camel_ews_store_summary_set_foreign_subfolders (CamelEwsStoreSummary *ews_summary,
+						const gchar *folder_id,
+						gboolean foreign_subfolders)
+{
+	S_LOCK (ews_summary);
+
+	g_key_file_set_boolean (
+		ews_summary->priv->key_file,
+		folder_id, "ForeignSubfolders", foreign_subfolders);
+	ews_summary->priv->dirty = TRUE;
+
+	S_UNLOCK (ews_summary);
+}
+
 void
 camel_ews_store_summary_store_string_val (CamelEwsStoreSummary *ews_summary,
                                           const gchar *key,
@@ -761,6 +780,23 @@ camel_ews_store_summary_get_foreign (CamelEwsStoreSummary *ews_summary,
 	return ret;
 }
 
+gboolean
+camel_ews_store_summary_get_foreign_subfolders (CamelEwsStoreSummary *ews_summary,
+						const gchar *folder_id,
+						GError **error)
+{
+	gboolean ret;
+
+	S_LOCK (ews_summary);
+
+	ret = g_key_file_get_boolean (
+		ews_summary->priv->key_file, folder_id, "ForeignSubfolders", error);
+
+	S_UNLOCK (ews_summary);
+
+	return ret;
+}
+
 gchar *
 camel_ews_store_summary_get_string_val (CamelEwsStoreSummary *ews_summary,
                                          const gchar *key,
@@ -818,6 +854,52 @@ camel_ews_store_summary_get_folders (CamelEwsStoreSummary *ews_summary,
 	return folders;
 }
 
+/* get list of folder IDs, which are foreign folders */
+GSList *
+camel_ews_store_summary_get_foreign_folders (CamelEwsStoreSummary *ews_summary,
+					     const gchar *prefix)
+{
+	GSList *folders = NULL;
+	gchar **groups = NULL;
+	gsize length;
+	gint prefixlen = 0;
+	gint i;
+
+	if (prefix)
+		prefixlen = strlen (prefix);
+
+	S_LOCK (ews_summary);
+
+	groups = g_key_file_get_groups (ews_summary->priv->key_file, &length);
+
+	S_UNLOCK (ews_summary);
+
+	for (i = 0; i < length; i++) {
+		if (!g_ascii_strcasecmp (groups[i], STORE_GROUP_NAME))
+			continue;
+
+		if (!camel_ews_store_summary_get_foreign (ews_summary, groups[i], NULL))
+			continue;
+
+		if (prefixlen) {
+			const gchar *fname;
+
+			fname = g_hash_table_lookup (
+				ews_summary->priv->id_fname_hash, groups[i]);
+
+			if (!fname || strncmp (fname, prefix, prefixlen) ||
+			    (fname[prefixlen] && fname[prefixlen] != '/'))
+				continue;
+		}
+
+		folders = g_slist_append (folders, g_strdup (groups[i]));
+	}
+
+	g_strfreev (groups);
+
+	return folders;
+}
+
 gboolean
 camel_ews_store_summary_remove_folder (CamelEwsStoreSummary *ews_summary,
                                        const gchar *folder_id,
