@@ -28,7 +28,6 @@
 #include <glib/gi18n-lib.h>
 #include <gtk/gtk.h>
 
-#include <libedataserverui/libedataserverui.h>
 #include <libemail-engine/e-mail-session.h>
 
 #include "camel/camel-ews-store.h"
@@ -154,6 +153,9 @@ add_foreign_folder_to_camel (CamelEwsStore *ews_store,
 	g_free (foreign_mailbox_id);
 	g_free (mailbox);
 
+	if (include_subfolders)
+		camel_ews_store_update_foreign_subfolders (ews_store, fid->id);
+
 	return TRUE;
 }
 
